/*
	$Id: outputsource_file.h,v 1.2 2001/04/03 20:10:07 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! clanCore="I/O Data"

#ifndef header_outputsource_file
#define header_outputsource_file

#include "outputsource.h"

#include <string>
#include <stdio.h>

class CL_OutputSource_File : public CL_OutputSource
//: Outputsource writing to file.
{
//!Construction:
public:
	CL_OutputSource_File(const char *);

	CL_OutputSource_File();

	virtual ~CL_OutputSource_File();
	
//!Attributes:
public:
	virtual int tell() const;
	//: Returns current position in output source.
	//!retval: Current position in source.

	virtual int size() const;
	//: Returns the size of the output source
	//!retval: Current size of output source.

	std::string get_data() const;
	//: Get the pointer to the internal data array in the memory source.
	//: Used to extract the data after usage.
	//!retval: Pointer to the data written.

//!Operations:
public:
	virtual void set_system_mode();
	//: Changes output data endianess to the local systems mode.

	virtual void set_big_endian_mode();
	//: Changes output data endianess to big endian mode.

	virtual void set_little_endian_mode();
	//: Changes output data endianess to little endian mode. This is the 
	//: default setting.

	virtual void write_int32(int data);
	//: Writes a signed 32 bit integer to output source.

	virtual void write_uint32(unsigned int data);
	//: Writes an unsigned 32 bit integer to output source.

	virtual void write_short16(short data);
	//: Writes a signed 16 bit integer (short) to output source.

	virtual void write_ushort16(unsigned short data);
	//: Writes an unsigned 16 bit integer (short) to output source.

	virtual void write_char8(char data);
	//: Writes a signed 8 bit integer (char) to output source.

	virtual void write_uchar8(unsigned char data);
	//: Writes an unsigned 8 bit integer (char) to output source.

	virtual void write_float32(float data);
	//: Writes an ieee format floating point to output source.

	virtual void write_bool8(bool data);
	//: Writes a bool to output source.

	virtual int write(const void *data, int size);
	//: Writes larger amounts of data (no endian and 64 bit conversion):
	//!param: data - Points to the array written.
	//!param: size - Number of bytes to write.
	//!retval: Num bytes actually written.
	
	virtual void open();
	//: Opens the output source. By default, it is open.
	
	virtual void close();
	//: Closes the output source.

	virtual CL_OutputSource *clone();
	//: Make a copy of the current outputsource, standing at the same position.
	//!retval: The copy of the output source.

	virtual void write_string(const char *string);
	//: Writes a string to the output source.
	// The binary format written to the output source is first an uint32 telling the length of the
	// string, and then the string itself.

//!Implementation:
private:
	unsigned int pos;
	FILE *file;
	std::string filename;
};

#endif
