/*
	$Id: rect.h,v 1.15 2001/04/03 08:15:18 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Rect class.

*/

//! clanCore="Math"

#ifndef header_rect
#define header_rect

class CL_Rect
{
public:
	int x1, y1;
	int x2, y2;
	
public:
	CL_Rect() : x1(0), y1(0), x2(0), y2(0) { }
	CL_Rect(int nx1, int ny1, int nx2, int ny2) : x1(nx1), y1(ny1), x2(nx2), y2(ny2) { }
	
	CL_Rect(const CL_Rect &rect)
	{
		this->x1 = rect.x1;
		this->x2 = rect.x2;
		this->y1 = rect.y1;
		this->y2 = rect.y2;
	};

	static CL_Rect center(int center_x, int center_y, int width, int height)
	{
		return CL_Rect(
			center_x - width/2,
			center_y - height/2,
			center_x + width/2,
			center_y + height/2);
	}

	static CL_Rect left(int x, int y, int width, int height, bool center_vert = true)
	{
		return CL_Rect(
			x - width,
			center_vert ? (y - height/2) : y,
			x,
			center_vert ? (y + height/2) : y);
	}

	static CL_Rect right(int x, int y, int width, int height, bool center_vert = true)
	{
		return CL_Rect(
			x,
			center_vert ? (y - height/2) : y,
			x + width,
			center_vert ? (y + height/2) : y);
	}

	bool operator == (const CL_Rect &rect) const;

	void move(int delta_x, int delta_y)
	{
		x1 += delta_x;
		x2 += delta_x;
		y1 += delta_y;
		y2 += delta_y;
	};

	bool inside(int x, int y) const
	{
		return x >= x1 && y >= y1 && x < x2 && y < y2;
	}

	int get_width() const { return x2 - x1; };
	int get_height() const { return y2 - y1; };

	void clear() { x1 = y1 = x2 = y2 = 0; };
};

#endif
