/*
	$Id: resource.h,v 1.9 2001/04/13 21:45:14 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! clanCore="Resources"

#ifndef header_resource
#define header_resource

#include "../../signals.h"
#include <string>
class CL_ResourceOptions;
class CL_ResourceManager;
class CL_ResourceData;
class CL_InputSource;
class CL_OutputSource;
class CL_InputSourceProvider;
class CL_OutputSourceProvider;

class CL_Resource
//: Interface representing a resource in the resource manager.
{
//!Construction:
public:
	CL_Resource(
		const std::string &type,
		const std::string &name,
		const std::string &location,
		const CL_ResourceOptions &options,
		const CL_ResourceManager &manager);

	CL_Resource(const CL_Resource &copy);

	CL_Resource();

	virtual ~CL_Resource();

//!Attributes:
public:
	const std::string &get_type() const;
	// Returns the type of the resource.

	const std::string &get_name() const;
	// Returns the name of the resource.

	const std::string &get_location() const;
	// Returns the location of the resource.

	CL_ResourceOptions &get_options();
	// Returns the resource options.

	CL_ResourceManager get_manager();
	// Returns the resource manager.

	CL_ResourceData *get_data(const std::string &name);
	// Returns the data with the matching name.

//!Signals:
public:
	CL_Signal_v0 &sig_load_file();
	// Signal invoked when a call to load_file is made.

	CL_Signal_v1<CL_InputSourceProvider *> &sig_load_datafile();
	// Signal invoked when a call to load_datafile is made.

	CL_Signal_v1<CL_OutputSourceProvider *> &sig_save_datafile();
	// Signal invoked when a call to save_datafile is made.

	CL_Signal_v0 &sig_unload();
	// Signal invoked when a call to unload is made.

//!Operations:
public:
	void operator = (const CL_Resource &copy);
	// Copy a resource.

	void attach_data(const std::string &name, CL_ResourceData *data);
	// Attach some data to the resource.

	void detach_data(CL_ResourceData *data);
	// Detach some data to the resource.

	void unload();
	// Unloads the resource from memory.

	void load();
	// Loads the resource, using the prefered source as specified by
	// the resource manager.

	void load_file();
	// Loads the resource from file.

	void load_datafile(CL_InputSourceProvider *input_provider);
	// Loads the resource from the input source.

	void save_datafile(CL_OutputSourceProvider *output_provider);
	// Saves the resource to the output source.

//!Implementation:
public:
	CL_Resource(class CL_Resource_Generic *impl);
	class CL_Resource_Generic *impl;
};

#endif
