/*
	$Id: resourcetype.h,v 1.7 2001/04/12 05:25:26 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! clanCore="Resources"

#ifndef header_resourcetype
#define header_resourcetype

#include <string>
#include <list>

class CL_Resource;

class CL_ResourceType
{
//!Construction:
public:
	CL_ResourceType(const std::string &type);
	// Constructs an registers a resource type.

	virtual ~CL_ResourceType();
	// Destructs and unregisters a resource type.

//!Attributes:
public:
	const std::string &get_type() const { return type; }
	// Returns the resource type name.

	static std::list<CL_ResourceType*> resource_types;
	// Global static list of all registered resource types.

//!Operations:
public:
	virtual void connect_data(CL_Resource &resource)=0;
	// The resource manager calls this function just after a resource object
	// has been created. The resource type should attach its resource data
	// to the resource in this function.

//!Implementation:
private:
	std::string type;
};

template<class Resource>
class CL_RegisterResourceType : public CL_ResourceType
// The CL_RegisterResourceType template class is a quick and convient way
// to register a resource type without doing a full inheritance of
// CL_ResourceType.
{
public:
	CL_RegisterResourceType(const std::string &type)
	: CL_ResourceType(type)
	{
	}

	virtual void connect_data(CL_Resource &resource)
	{
		new Resource(resource); // attach resource data.
	}
};

#endif
