
//! clanCore="System"

#ifndef header_timer
#define header_timer

#include "keep_alive.h"
#include "../../signals.h"

class CL_Timer : public CL_KeepAlive
//: Timer signal emitter.
// Use this class to emit a signal as a timer event.
{
public:
	CL_Timer(unsigned long interval = 1000);
	// Constructs and registers the timer as a keep_alive listener.
	//!param: interval - Interval in millisecs between each signal emit.
	
	virtual void keep_alive();
	// Called when CL_System::keep_alive() is called.

	CL_Signal_v0 sig_timer;
	// Signal emitted everytime the interval has elapsed.
	
	void set_interval(unsigned long interval);
	// Sets a new interval for the timer.
	//!param: interval - Interval in millisecs between each signal emit.

	unsigned long get_interval();
	// Returns the current time interval.
	//!retval: The current time interval.

	void enable();
	// Call this function to enable the timer.
	
	void disable();
	// Call this function to disable the timer.
	
private:
	unsigned long interval;
	unsigned long last_time;
	bool enabled;
};

#endif
