/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_combobox
#define header_combobox

#include "component.h"

class CL_ComboBox : public CL_Component
//: Combobox component
{
public:
//!Construction:
	CL_ComboBox(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_ComboBox(
		const CL_Rect &pos,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	virtual ~CL_ComboBox();

//!Attributes:
public:
	int get_count() const;
	//: Returns the number of items in the combobox.

	std::list<std::string> &get_items() const;
	//: Returns all items in the list.

	std::string get_current_text() const;
	//: Returns the current text item. 

	std::string get_text(int index) const;
	//: Returns the text item at a given index.

	int get_current_item() const;
	//: Returns the index of the current combobox item.

	bool is_selected(int index) const;
	//: Returns true if item index is selected. Returns false if it is not selected or if there is an error.

//!Operations:
public:
	int insert_item(const std::string &text, int index = -1);
	//: Inserts a text item at position index. 
	// The item will be appended if index is negative.

	void remove_item(int index);
	//: Removes and deletes the item at position index.
	// If index is equal to currentItem(), a new item gets highlighted and the highlighted() signal is emitted. 

	void change_item(const std::string &text, int index);
	//: Replaces the item at position index with a text.

	void set_current_item(int index);
	//: Sets the current combobox item. This is the item to be displayed on the combobox button. 

	void clear_selection();
	//: Deselect current selected item.

	void sort(bool ascending = true);
	//: Sorts the items in ascending order, if ascending is true, or descending otherwise. 

	void clear();
	//: Removes all combobox items. 

//!Signals:
public:
	CL_Signal_v1<int> &sig_activated();
	//: This signal is emitted when a new item has been activated (selected).
	//!param: int index - The position of the item in the popup list. 

	CL_Signal_v1<int> &sig_highlighted();
	//: This signal is emitted when a new item has been highlighted.
	//!param: int index - The position of the item in the popup list. 

//!Implementation:
private:
	CL_ComboBox(const CL_ComboBox &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	class CL_ComboBox_Generic *impl;
};  

#endif

