/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Framework"

#ifndef header_gui_manager
#define header_gui_manager

#include <stddef.h>

#include "component.h"

class CL_ComponentManager;
class CL_StyleManager;

class CL_GUIManager : public CL_Component
//: Manages the GUI; root component.
{
//!Construction:
public:
	CL_GUIManager(CL_StyleManager *style);

	CL_GUIManager(CL_Component *parent, CL_StyleManager *style);
	
	virtual ~CL_GUIManager();

//!Attributes:
public:
	CL_Component *get_focus();
	//: Returns the currently focused component.

//!Operations:
public:
	void set_focus(CL_Component *component);
	//: Sets the focus component of the gui.

	void show();
	//: Draws the GUI once.

	void run();
	//: Redraws the GUI continuosly.

	void quit();
	//: Break run loop.

	void enable_input();
	//: Tells the GUI to start accepting input.
	
	void disable_input();
	//: Tells the GUI to stop accepting input.

	void gui_capture_mouse(CL_Component *component);
	//: Sets the GUI in capture mode. [should we move this out of API space? -- mbn]

	void gui_release_mouse();
	//: Takes GUI out of capture mode. [should we move this out of API space? -- mbn]

//!Implementation:
private:
	class CL_GUIManager_Generic *impl;
};

#endif
