/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_listbox
#define header_listbox

#include "component.h"

class CL_ListBox : public CL_Component
//: Listbox component
{
//!Construction:
public:
	CL_ListBox(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);
	
	CL_ListBox(
		const CL_Rect &pos,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	virtual ~CL_ListBox();

//!Attributes:
public:
	CL_Component *get_client_area() const;
	//: Returns the client area of the component.

	int get_count() const;
	//: Returns the number of items in the list box. 

	std::list<std::string> &get_items() const;
	//: Returns all items in the list.

	std::string get_current_text() const;
	//: Returns the text of the current item.

	std::string get_text(int index) const;
	//: Returns the text at position index.

	int get_current_item() const;
	//: Returns the index of the current item of the listbox, or -1 if no item has been selected. 

	bool is_selected(int index) const;
	//: Returns true if item index is selected. Returns false if it is not selected or if there is an error.

	int get_item_height() const;
	//: Returns the height of one item.

	int get_top_item() const;
	//: Returns the index of the item at the top of the screen.

	int get_max_visible_items() const;
	//: Returns number of items maximum visible in listbox (without scrolling)

	bool has_scrollbar() const;
	//: Returns true if a scrollbar is visible in the listbox.

//!Operations:
public:
	int insert_item(const std::string &text, int index = -1);
	//: Inserts text into the list at index. 
	// If index is negative, text is inserted at the end of the list.
	// Returns the index of the text

	void remove_item(int index);
	//: Removes and deletes the item at position index.
	// If index is equal to get_current_item(), a new item gets highlighted and the sig_highlighted() signal is emitted. 

	void change_item(const std::string &text, int index);
	//: Replaces the item at position index with text. 

	void set_current_item(int index);
	//: Sets the highlighted item to the item index.
	// The highlighting is moved and the listbox scrolled as necessary. 

	void clear_selection();
	//: Deselects current selected item.

	void set_item_height(int height);
	//: Sets the height of each item.

	void set_max_visible_items(int count);
	//: Sets the number of items maximum visible in listbox (without scrolling)

	void set_top_item(int index);
	//: Scrolls the list box so the item at position index in the list is displayed in the top row of the listbox. 

	void sort(bool ascending = true);
	//: Sorts the items in ascending order, if ascending is true, or descending otherwise. 

	void clear();
	//: Deletes all items in the list. 

//!Signals:
public:
	CL_Signal_v1<int> &sig_highlighted();
	//: This signal is emitted when the user highlights a new current item.
	// The argument is the index of the new item, which is already current. 

	CL_Signal_v1<int> &sig_activated();
	//: This signal is emitted when the user clicks on an item or presses return when an item is highlighted.
	// The argument is the index of the selected item. 

//!Implementation:
private:
	CL_ListBox(const CL_ListBox &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	class CL_ListBox_Generic *impl;
};

#endif
