/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_menudata
#define header_menudata

#include "component.h"

class CL_MenuData : public CL_Component
//: MenuData
{
//!Construction:
public:
	CL_MenuData(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);
	
	virtual ~CL_MenuData();

//!Attributes:
public:
	unsigned int get_count() const;
	//: Return number of items

	const std::string &get_text(int id) const;
	//: Return item text

	bool is_item_enabled(int id) const;

	bool is_item_checked(int id) const;
	
	int get_index(int id) const;
	//: Returns item position

	int get_id(int index) const;
	//: Returns item id

//!Operations:
public:
	int insert_item(const std::string &text, int id = -1, int index = -1);

	int insert_separator(int id = -1, int index = -1);

	void remove_item(int id);
	//: Remove item

	void remove_item_at(int index);
	//: Remove item as position index

	void clear();
	//: Remove all items

	void change_item(const std::string &text, int id);
	//: Change item text

	void enable_item(int id, bool enable);
	
	void check_item(int id, bool check);

	void set_id(int index, int id);
	//: Sets item id

//!Implementation:
private:
	CL_MenuData(const CL_MenuData &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	class CL_MenuData_Generic *impl;
};  

#endif
