/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_menuitem
#define header_menuitem

#include "component.h"

class CL_MenuItem : public CL_Component
//: MenuItem component
{
//!Construction:
public:
	CL_MenuItem(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_MenuItem(
		const std::string &text,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_MenuItem(
		const std::string &text,
		bool checked,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_MenuItem(
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	virtual ~CL_MenuItem();

//!Attributes:
public:
	bool is_checkable() const;

	bool is_separator() const;

	bool is_checked() const;

	bool is_highlighted() const;

	bool has_submenu() const;
	
	const std::string &get_text() const;
	//: Returns the item label.
	//!retval: The item label.

//!Operations:
public:
	virtual void add_child(CL_Component *child);

	void set_text(const std::string &text);
	//: Sets the item label.
	//!param: text - The item label.

//!Signals:
public:
	CL_Signal_v0 &sig_clicked();
	//: This signal is emitted when the item is clicked on.

//!Implementation:
private:
	CL_MenuItem(const CL_MenuItem &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	class CL_MenuItem_Generic *impl;
};

#endif
