/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_messagebox
#define header_messagebox

#include "component.h"
#include "window.h"

class CL_MessageBox : public CL_Window
//: MessageBox component
{
//!Construction:
public:
	CL_MessageBox(
		const CL_ComponentOptions &options,	
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_MessageBox(
		const std::string &title,
		const std::string &text,
		const std::string &button1,
		const std::string &button2,
		const std::string &button3,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	virtual ~CL_MessageBox();

	static void info(
		const std::string &text,
		CL_Component *parent);
	// Creates a messagebox "Information" with text and an OK button.

	static void info(
		const std::string &title,
		const std::string &text,
		CL_Component *parent);
	// Creates a message box with title, text and an OK button.

	static int info(
		const std::string &title,
		const std::string &text,
		const std::string &button1,
		const std::string &button2,
		const std::string &button3,
		CL_Component *parent);
	// Creates a message box with title, text and up to three buttons.

//!Attributes:
public:
	const std::string &get_text() const;
	//: Returns the message text.

	int get_result_button() const;
	//: Returns the index of the button clicked.

//!Operations:
public:
	void set_text(const std::string &text);
	//: Sets the message text.

//!Signals:
public:
	CL_Signal_v0 &sig_button1();
	CL_Signal_v0 &sig_button2();
	CL_Signal_v0 &sig_button3();

//!Implementation:
private:
	// TODO:Fix this copy constructor:
//	CL_MessageBox(const CL_MessageBox &copy) : CL_Window(NULL, NULL) { return; } // disallow copy construction.
	class CL_MessageBox_Generic *impl;
};

#endif
