/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_scrollbar
#define header_scrollbar

#include "component.h"

class CL_Button;
class CL_ComponentOptions;
class CL_Rect;

class CL_ScrollBar : public CL_Component
//: ScrollBar component
{
//!Construction:
public:
	CL_ScrollBar(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_ScrollBar(
		const CL_Rect &pos,
		int min,
		int max,
		bool orientation,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	virtual ~CL_ScrollBar();

//!Attributes:
public:
	CL_Component *get_client_area() const;
	//: Returns the client area of the component.

	bool is_vertical() const;
	//: Returns the orientation of the scroll bar.
	// Return true if scrollbar is vertical, false otherwise.

	bool is_tracking() const;
	//: Returns true if tracking is enabled, or false if tracking is disabled. 
	// Tracking is initially enabled. See set_tracking for more info on tracking.

	int get_min_value() const;
	//: Returns the current min value.

	int get_max_value() const;
	//: Returns the current max value.

	int get_range() const;
	//: Returns the range (delta max-min).

	int get_value() const;
	//: Returns the current range control value. 

	bool is_dragging_slider() const;
	//: Returns true if the user has clicked the mouse on the slider and is currently dragging it, or false if not. 

	bool is_fixed_length_slider() const;
	//: Returns true if this is a fixed length slider, false otherwise.

	CL_Rect &get_slider_rect() const;
	//: Returns the scroll bar slider rectangle. 

//!Operations:
public:
	void set_fixed_length_slider(bool fixed_length = true);
	//: Sets the length-mode of the slider.
	// If fixed_length is true, the slider is constant in size. (See set_slider_length()).
	// If fixed_length is false, the slider is resized automatically based on the range.

	void set_slider_length(int length);
	//: Sets the current size of the slider.
	// Only usable in fixed-length sliders.

	void set_vertical(bool enable = true);
	//: Sets the scroll bar orientation to vertical if enable is true, horizontal otherwise.

	void set_tracking(bool enable = true);
	//: Enables scroll bar tracking if enable is true, or disables tracking if enable is false. 
	// If tracking is enabled (the default), the scroll bar emits the
	// sig_value_changed() signal while the slider is being dragged. If tracking is
	// disabled, the scroll bar emits the valueChanged() signal only when the user
	// releases the mouse button after moving the slider.

	void set_range(int min_value, int max_value);
	//: Sets the range between min_value and max_value. 
	// If max_value is smaller than min_value, max_value is set to min_value.

	void set_min_value(int value);
	//: Sets the range to start from value. 
	// If current maximum value is less than the minvalue, maxvalue is set to minvalue.

	void set_max_value(int value);
	//: Sets the range to end at value. 
	// If current maximum value is less than the minvalue, maxvalue is set to minvalue.

	void set_value(int value);
	//:: Sets the range controls value to value and forces it to be within the legal range. 

	void increase(int step = 1);
	//: Increases current value with step.

	void decrease(int step = 1);
	//: Decreases current value with step.

/*
	int get_drag_offset_x() const;

	int get_drag_offset_y() const;

	const CL_Rect &get_drag_rect() const;

	bool is_scrolling() const;

	bool is_section_scrolling() const;

	int get_scroll_direction() const;

	void set_drag_offset_x(int new_offset_x);

	void set_drag_offset_y(int new_offset_y);

	void set_drag_rect(const CL_Rect &new_rect);
*/
//!Signals:
public:
	CL_Signal_v1<int> &sig_value_changed();
	//: This signal is emitted when the scroll bar value has changed, with the new scroll bar value as an argument. 
	// See set_tracking() for specifics on this signal.

	CL_Signal_v0 &sig_slider_pressed();
	//: This signal is emitted when the user presses the slider with the mouse. 

	CL_Signal_v1<int> &sig_slider_moved();
	//: This signal is emitted when the slider is moved by the user, with the new scroll bar value as an argument. 
	// This signal is emitted even when tracking is turned off. 

	CL_Signal_v0 &sig_slider_released();
	//: This signal is emitted when the user releases the slider with the mouse. 

//!Implementation:
private:
	CL_ScrollBar(const CL_ScrollBar &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	class CL_ScrollBar_Generic *impl;
};

#endif
