/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_stylemanager_default
#define header_stylemanager_default

#include "stylemanager.h"
#include "component_options.h"

class CL_ResourceManager;
class CL_Component;

enum GUIColor
{
	GUICOLOR_SELECTED_OUTLINE,
	GUICOLOR_DARK_OUTLINE,
	GUICOLOR_DARK_OUTLINE_DISABLED,
	GUICOLOR_DARKER_SHADE,
	GUICOLOR_MEDIUM_SHADE,
	GUICOLOR_BRIGHT_SHADE,
	GUICOLOR_BRIGHT_SHADE_DISABLED,
	GUICOLOR_DARK_SHADE,
	GUICOLOR_DARK_SHADE_DISABLED,
	GUICOLOR_WHITE,
	GUICOLOR_SELECTION,
	GUICOLOR_CARET,
	GUICOLOR_SCROLLBAR,
	GUICOLOR_BUTTON,
	GUICOLOR_BUTTON_DISABLED,
	GUICOLOR_BUTTON_TOGGLED,
	GUICOLOR_WINDOW_NORMAL,
	GUICOLOR_WINDOW_TITLEBAR,
	GUICOLOR_PROGRESSBAR,
	GUICOLOR_PROGRESSBAR_FILLED
};

class CL_StyleManager_Default : public CL_StyleManager
//: Style manager that uses the default clanlib gui style.
{
// Construction:
public:
	CL_StyleManager_Default(CL_ResourceManager *resources);
	virtual ~CL_StyleManager_Default();

// Attributes:
public:
	const std::string &get_default_font() const;

	const std::string &get_combo_normal() const;
	const std::string &get_combo_toggled() const;
	const std::string &get_combo_disabled() const;

	const std::string &get_checkbox_checked() const;
	const std::string &get_checkbox_unchecked() const;
	const std::string &get_checkbox_disabled() const;

// Operations:
public:
	void set_default_font(const std::string &font);

	void set_combo_normal(const std::string &normal);
	void set_combo_toggled(const std::string &toggled);
	void set_combo_disabled(const std::string &disabled);

	void set_checkbox_checked(const std::string &checked);
	void set_checkbox_unchecked(const std::string &unchecked);
	void set_checkbox_disabled(const std::string &disabled);

	void fill_rect(int x1, int y1, int x2, int y2, GUIColor col);
	void draw_rect(int x1, int y1, int x2, int y2, GUIColor col);
	void draw_line(int x1, int y1, int x2, int y2, GUIColor col);
	void draw_box(int x1, int y1, int x2, int y2, GUIColor topleft, GUIColor bottomright);

// Overrideables:
public:
	virtual CL_Component *create_component(
		const std::string &type,
		const CL_ComponentOptions &options,
		CL_Component *parent);

	virtual void connect_styles(
		const std::string &type,
		const CL_ComponentOptions &options,
		CL_Component *owner);

// Implementation:
private:
	CL_StyleManager_Default(const CL_StyleManager_Default &copy) : CL_StyleManager(NULL) { return; } // disallow copy contruction
	class CL_StyleManager_Default_Generic *impl;
};

#endif
