/*
	$Id: streamed_mikmod_sample.h,v 1.10 2001/04/28 12:04:36 grumbel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Streamed sample header file
*/

//! clanMikMod="Sound Providers"

#ifndef header_streamed_mikmod_sample
#define header_streamed_mikmod_sample

#include <stdlib.h>
#include "../core.h"
#include "../Sound/stream_soundprovider.h"
#include "../Sound/soundbuffer.h"


#include <mikmod.h>




class CL_Streamed_MikModSample : public CL_StreamSoundProvider
//: CL_Streamed_MikModSample is used to store MikMod modules in memory.
// This class stores the ClanLib sample the way the MikMod MODULE struct
// does. It's only intended to store the module in memory, not to play
// it directly. To play the module, use CL_Streamed_MikModSample_Session
// instead.
{
protected:
	std::string filename;
	bool looped;
	bool loaded_from_resource;
	CL_ResourceManager *manager;

	MODULE *module;
public:
	static CL_SoundBuffer *create(
		const char *filename, 
		CL_InputSourceProvider *inputprovider = NULL, 
		bool looped = false);
	//: Use this function to create a module from a file for instance
	static CL_SoundBuffer *load(
		const char *res_id, 
		CL_ResourceManager *manager, 
		bool looped = false);
	//: Loads a module from a resource file (.scr or .dat)

	CL_Streamed_MikModSample(
		const char *filename, 
		CL_InputSourceProvider *inputprovider = NULL, 
		bool looped = false);
	//: Use this function to create a module from a file for instance
	CL_Streamed_MikModSample(
		const char *res_id, 
		CL_ResourceManager *manager, 
		bool looped = false);
	//: Loads a module from a resource file (.scr or .dat)
	virtual ~CL_Streamed_MikModSample();

	virtual CL_StreamSoundProvider_Session *begin_session();
	//: Starts playing the module

	virtual void end_session(CL_StreamSoundProvider_Session *session);
	//: Ends the session

};

class CL_Streamed_MikModSample_Session : public CL_StreamSoundProvider_Session
{
//: CL_Streamed_MikModSample_Session is used to play and control a module.
//
protected:
	friend class CL_Streamed_MikModSample;

	CL_Streamed_MikModSample_Session(MODULE *_module, bool _looped);

	MODULE *module;
	bool looped;

	SoundFormat sample_format;
	int sample_size;
	int sample_freq;
	int sample_left;

public:
/*
	CL_Streamed_MikModSample_Session::CL_Streamed_MikModSample_Session(
	CL_InputSource *_input, bool _looped, const char* filename)
*/
	virtual ~CL_Streamed_MikModSample_Session();

	virtual void stop();
	virtual bool play();
	virtual bool set_position(int pos);
	
	virtual bool eof() const;
	virtual int get_data(void *data_ptr, int data_requested);
	virtual int get_frequency() const;
	virtual SoundFormat get_format() const;
	
	static BOOL clanMikMod_IsPresent () { return 1; } 
	static void clanMikMod_Update () { /* do nothing */ } 
};

#endif

















