
#ifndef header_buffered_socket
#define header_buffered_socket

//! clanNetwork="Sockets"

#include "socket.h"

class CL_BufferedSocket : public CL_Socket
//: Non-blocked, buffered socket transmission interface.
// The buffered socket class works like a socket in non-blocking mode, with a few exceptions:
// <ul>
// <li>any write operation will always write the full data. Any data not immidiately writeable
// to the socket will be buffered and written in a seperate thread as the socket can receive
// them. When all the data has been written, the write event will trigger.</li>
// 
// <li>incoming data is also buffered. Each time the input buffer has been filled (which
// can be changed with set_input_buffer_size(int size), the read event will trigger.</li>
// </ul>
// Using this class, you can easilly write and read data in the chunk sizes you prefer.
{
//!Construction:
public:
	CL_BufferedSocket(int socket);
	// Constructs an attached socket.

	CL_BufferedSocket(Type type);
	// Constructs a new socket using the specified protocol type.

	CL_BufferedSocket(const CL_Socket &socket);
	// Create socket that is attached to the passed socket.
	// Note that since the buffered socket class will read data from the socket (buffering it),
	// it will no longer be safe to use the original socket until the buffered socket instance
	// has been destroyed.

	CL_BufferedSocket(const CL_BufferedSocket &copy);
	// Copy constructor.

	virtual ~CL_BufferedSocket();

//!Attributes:
public:
	int get_input_buffer_size() const;
	// Returns the size of the buffered input buffer. Each time it is full,
	// the read event will trigger.

//!Operations:
public:
	void set_input_buffer_size(int size);
	// Sets the size of the buffered input buffer. Each time it is full,
	// the read event will trigger.

//!Implementation:
private:
	class CL_BufferedSocket_Generic *impl;
};

#endif
