
#ifndef header_ip_address
#define header_ip_address

//! clanNetwork="Sockets"

#include <string>

class CL_IPAddress
//: Socket name; container class for an IP address and port.
{
//!Construction:
public:
	CL_IPAddress();
	// Constructs an IP address using INADDR_ANY and port 0.

	CL_IPAddress(unsigned int address, unsigned short port);
	// Constructs an IP address using the passed address and port.

	CL_IPAddress(const std::string &hostname, unsigned short port);
	// Construct an IP address using a hostname.

	CL_IPAddress(const CL_IPAddress &copy);
	// Copy constructor.

//!Attributes:
public:
	unsigned int get_address() const { return ip; }
	// Returns the IP address in network byte order.

	unsigned short get_port() const { return port; }
	// Returns the IP port in host byte order.

	unsigned char &operator[](int i) { return ((unsigned char*) &ip)[i]; }
	// Returns the specified address field.

	const unsigned char &operator[](int i) const { return ((unsigned char*) &ip)[i]; }
	// Returns the specified address field.

//!Operations:
public:
	void set_address(unsigned int new_ip) { ip = new_ip; }
	// Set the IP address, in network byte order.

	void set_address(const std::string &hostname);
	// Set the IP address using a hostname.

	void set_port(unsigned short new_port) { port = new_port; }
	// Set the IP port.

	std::string dns_lookup() const;
	// Do a DNS lookup on the IP address.

//!Implementation:
private:
	unsigned int ip;
	// IP address in network byte order.

	unsigned short port;
	// IP port in host byte order.
};

#endif
