/*
	$Id: netobject.h,v 1.8 2001/03/17 07:29:34 vaidhy Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_netobject
#define header_netobject

//! clanNetwork="NetObjects"

#include <string>
#include "../signals.h"

class CL_NetObject
//: Network replication object.
//
// The netobject is a message dispatcher for objects being replicated over the network.
// It provides an easy system for a server object to send messages to replicated clients
// on remote computers, as well as getting answers back.
{
//!Construction:
public:
	CL_NetObject(const CL_NetObject &copy);
	// Copy constructor.

	CL_NetObject(class CL_NetObjectChannel *obj_channel);
	// Construct a new netobject, assigning it an unique handle that identifies it over the
	// network. Use this constructor to create a server side netobject.

	CL_NetObject(int obj_id, class CL_NetObjectChannel *obj_channel);
	// Construct a netobject, assigning it the object handle obj_id. This is normally used
	// to construct a client side netobject.

	virtual ~CL_NetObject();

//!Attributes:
	int get_obj_id() const;
	// Returns the netobject handle that identifies it over the network.

//!Operations:
	CL_Slot connect(int msg_type, CL_Slot_v1<class CL_InputSource &> *slot);
	// Connect the slot to the specified message sub-type.

	CL_Slot connect(int talkback_type, CL_Slot_v2<const class CL_NetComputer &, class CL_InputSource &> *slot);
	// Connect the slot to the specified talkback message sub-type.

	void send(int msg_type, const std::string &message);
	// Send a message to the client objects.

	void talkback(int talkback_type, const std::string &message);
	// Send a message to the server objects.

//!Implementation:
public:
	class CL_NetObject_Generic *impl;
};

#endif
