/*
	$Id: netobject_channel.h,v 1.7 2001/03/15 12:14:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_netobject_channel
#define header_netobject_channel

//! clanNetwork="NetObjects"

#include <string>
#include "../signals.h"

class CL_NetObjectChannel
//: Network object replication channel.
//
// Manages a netchannel with netobject replication data. It dispatches incoming
// messages to the correct netobject.
{
//!Construction:
public:
	CL_NetObjectChannel(const CL_NetObjectChannel &copy);
	// Copy constructor.

	CL_NetObjectChannel(class CL_NetSession *netgame, int channel);
	// Constructs a netobject channel.

	virtual ~CL_NetObjectChannel();

//!Attributes:
public:
	CL_Signal_v3<const class CL_NetObject &, int, const std::string &> &sig_create_object();
	// CL_Signal_v3<netobject, msgtype, message>.
	//
	// Signal that is called whenever the netobject channel receives a message for a netobject
	// it doesnt know. The application should hook a slot into this signal and determine what
	// to do with the message.

	void begin_sync(const class CL_NetGroup *group = NULL);
	// Sends a begin_sync message to the netchannel. This is used together with end_sync to
	// mark a group of objects. The client can use wait_sync to wait for a full transfer of
	// a list of objects.

	void end_sync(const class CL_NetGroup *group = NULL);
	// Sends a end_sync message to the netchannel. This is used together with end_sync to
	// mark a group of objects. The client can use wait_sync to wait for a full transfer of
	// a list of objects.

	bool wait_sync(int timeout = -1);
	// Waits for a synced group of objects to arrive. If the whole group havnt been received
	// before the timeout, the function will return false.

public:
	class CL_NetObjectChannel_Generic *impl;
};

#endif
