/*
	$Id: netvariables.h,v 1.5 2001/03/05 10:00:51 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_netvariables
#define header_netvariables

//! clanNetwork="NetObjects"

class CL_NetVariables
//: Easy serialization of a data structure.
//
// The netvariables class is used to read a set of variables from an object,
// and write them structured, in network byte order, to an output source.
//
// The Netvariables interface can also read the data again, and produce 'diff'
// packages where it only write variables that have changed since last write.
{
//!Construction:
public:
	CL_NetVariables();
	// Construct a netvariables object.

	virtual ~CL_NetVariables();

//!Attributes:
	bool is_different();
	// Returns true if the variables have changed since last save.

//!Operations:
	void add_bool(bool *var, int array = 1);
	// Add booleans to the data structure.

	void add_int(int *var, int array = 1);
	// Add integers to the data structure.

	void add_short(short *var, int array = 1);
	// Add shorts to the data structure.

	void add_float(float *var, int array = 1);
	// Add floats to the data structure.

	void add_double(double *var, int array = 1);
	// Add doubles to the data structure.

	void add_vars(CL_NetVariables *variables, int array = 1);
	// Add structures to the data structure.

	void save_all(class CL_OutputSource *msg);
	// Write all variables in the data structure to the output source.

	void load_all(class CL_InputSource *msg);
	// Read all variables in the data structure from the input source.

	void save_diff(class CL_OutputSource *msg);
	// Write all variables that have changed since last save.

	void load_diff(class CL_InputSource *msg);
	// Read a diff produced with save_diff from the input source.

public:
	class CL_NetVariables_Generic *impl;
};

#endif
